@@ -91,6 +91,18 @@ xsltParseStylesheetImport(xsltStylesheetPtr style, xmlNodePtr cur) {
 	goto error;
     }
 
+    res = style;
+    while (res != NULL) {
+        if (res->doc == NULL)
+	    break;
+	if (xmlStrEqual(res->doc->URL, URI)) {
+	    xsltTransformError(NULL, style, cur,
+	       "xsl:import : recursion detected on imported URL %s\n", URI);
+	    goto error;
+	}
+	res = res->parent;
+    }
+
     /*
      * Security framework check
      */
@@ -107,16 +119,19 @@ xsltParseStylesheetImport(xsltStylesheetPtr style, xmlNodePtr cur) {
 	}
     }
 
-    import = xmlParseFile((const char *)URI);
+#ifdef XSLT_PARSE_OPTIONS
+    import = xmlReadFile((const char *) URI, NULL, XSLT_PARSE_OPTIONS);
+#else
+    import = xmlParseFile((const char *) URI);
+#endif
     if (import == NULL) {
 	xsltTransformError(NULL, style, cur,
 	    "xsl:import : unable to load %s\n", URI);
 	goto error;
     }
 
-    res = xsltParseStylesheetImportedDoc(import);
+    res = xsltParseStylesheetImportedDoc(import, style);
     if (res != NULL) {
-	res->parent = style;
 	res->next = style->imports;
 	style->imports = res;
 	xmlHashScan(res->templatesHash, 
@@ -156,6 +171,7 @@ xsltParseStylesheetInclude(xsltStylesheetPtr style, xmlNodePtr cur) {
     xmlChar *uriRef = NULL;
     xmlChar *URI = NULL;
     xsltDocumentPtr include;
+    xsltDocumentPtr docptr;
 
     if ((cur == NULL) || (style == NULL))
 	return (ret);
@@ -175,6 +191,20 @@ xsltParseStylesheetInclude(xsltStylesheetPtr style, xmlNodePtr cur) {
 	goto error;
     }
 
+    /*
+     * in order to detect recursion, we check all previously included
+     * stylesheets.
+     */
+    docptr = style->includes;
+    while (docptr != NULL) {
+        if (xmlStrEqual(docptr->doc->URL, URI)) {
+	    xsltTransformError(NULL, style, cur,
+	        "xsl:include : recursion detected on included URL %s\n", URI);
+	    goto error;
+	}
+	docptr = docptr->includes;
+    }
+
     include = xsltLoadStyleDocument(style, URI);
     if (include == NULL) {
 	xsltTransformError(NULL, style, cur,
@@ -184,7 +214,11 @@ xsltParseStylesheetInclude(xsltStylesheetPtr style, xmlNodePtr cur) {
 
     oldDoc = style->doc;
     style->doc = include->doc;
+    /* chain to stylesheet for recursion checking */
+    include->includes = style->includes;
+    style->includes = include;
     ret = (int)xsltParseStylesheetProcess(style, include->doc);
+    style->includes = include->includes;
     style->doc = oldDoc;
     if (ret == 0) {
 		ret = -1;
