@@ -189,6 +189,28 @@ xsltFreeCompMatchList(xsltCompMatchPtr comp) {
     }
 }
 
+/**
+ * xsltNormalizeCompSteps:
+ * @payload: pointer to template hash table entry
+ * @data: pointer to the stylesheet
+ * @name: template match name
+ *
+ * This is a hashtable scanner function to normalize the compiled
+ * steps of an imported stylesheet.
+ */
+void xsltNormalizeCompSteps(void *payload,
+        void *data, const xmlChar *name ATTRIBUTE_UNUSED) {
+    xsltCompMatchPtr comp = payload;
+    xsltStylesheetPtr style = data;
+    int ix;
+
+    for (ix = 0; ix < comp->nbStep; ix++) {
+        comp->steps[ix].previousExtra += style->extrasNr;
+        comp->steps[ix].indexExtra += style->extrasNr;
+        comp->steps[ix].lenExtra += style->extrasNr;
+    }
+}
+
 /**
  * xsltNewParserContext:
  * @style:  the stylesheet
@@ -267,7 +289,7 @@ xsltCompMatchAdd(xsltParserContextPtr ctxt, xsltCompMatchPtr comp,
 	    xsltAllocateExtra(ctxt->style);
     }
     if (op == XSLT_OP_PREDICATE) {
-	comp->steps[comp->nbStep].comp = xmlXPathCompile(value);
+	comp->steps[comp->nbStep].comp = xsltXPathCompile(ctxt->style, value);
     }
     comp->nbStep++;
     return (0);
@@ -634,7 +656,8 @@ xsltTestCompMatch(xsltTransformContextPtr ctxt, xsltCompMatchPtr comp,
 			    xmlXPathFreeObject(newlist);
 			    return(-1);
 			}
-			
+			index = 0;
+
 			if ((parent == NULL) || (node->doc == NULL))
 			    nocache = 1;
 			else {
@@ -644,6 +667,7 @@ xsltTestCompMatch(xsltTransformContextPtr ctxt, xsltCompMatchPtr comp,
 					     BAD_CAST " fake node libxslt")))
 				nocache = 1;
 			}
+			
 			if (nocache == 0) {
 			    if (list != NULL)
 				xmlXPathFreeObject(list);
@@ -653,6 +677,8 @@ xsltTestCompMatch(xsltTransformContextPtr ctxt, xsltCompMatchPtr comp,
 				(void *) list;
 			    XSLT_RUNTIME_EXTRA(ctxt, select->previousExtra) =
 				(void *) doc;
+			    XSLT_RUNTIME_EXTRA(ctxt, select->indexExtra) =
+			        0;
 			    XSLT_RUNTIME_EXTRA_FREE(ctxt, select->lenExtra) =
 				(xmlFreeFunc) xmlXPathFreeObject;
 			} else
@@ -710,11 +736,11 @@ xsltTestCompMatch(xsltTransformContextPtr ctxt, xsltCompMatchPtr comp,
 			while (sibling != NULL) {
 			    if (sibling == previous)
 				break;
-			    if ((node->type == XML_ELEMENT_NODE) &&
-				(node->name != NULL) &&
+			    if ((previous->type == XML_ELEMENT_NODE) &&
+				(previous->name != NULL) &&
 				(sibling->name != NULL) &&
-				(node->name[0] == sibling->name[0]) &&
-				(xmlStrEqual(node->name, sibling->name))) {
+				(previous->name[0] == sibling->name[0]) &&
+				(xmlStrEqual(previous->name, sibling->name))) {
 				if ((select->value2 == NULL) ||
 				    ((sibling->ns != NULL) &&
 				     (xmlStrEqual(select->value2,
@@ -1001,7 +1027,7 @@ xsltTestCompMatchList(xsltTransformContextPtr ctxt, xmlNodePtr node,
 #define CUR_PTR ctxt->cur
 
 #define SKIP_BLANKS 							\
-    while (IS_BLANK(CUR)) NEXT
+    while (IS_BLANK_CH(CUR)) NEXT
 
 #define CURRENT (*ctxt->cur)
 #define NEXT ((*ctxt->cur) ?  ctxt->cur++: ctxt->cur)
@@ -1617,6 +1643,7 @@ xsltCompileLocationPathPattern(xsltParserContextPtr ctxt) {
 	 */
 	NEXT;
 	NEXT;
+	ctxt->comp->priority = 0.5;	/* '//' means not 0 priority */
 	xsltCompileRelativePathPattern(ctxt, NULL);
     } else if (CUR == '/') {
 	/*
@@ -1703,7 +1730,7 @@ xsltCompilePattern(const xmlChar *pattern, xmlDocPtr doc,
     current = end = 0;
     while (pattern[current] != 0) {
 	start = current;
-	while (IS_BLANK(pattern[current]))
+	while (IS_BLANK_CH(pattern[current]))
 	    current++;
 	end = current;
 	level = 0;
@@ -1758,6 +1785,11 @@ xsltCompilePattern(const xmlChar *pattern, xmlDocPtr doc,
 			 "xsltCompilePattern : parsing '%s'\n",
 			 element->pattern);
 #endif
+	/*
+	 Preset default priority to be zero.
+	 This may be changed by xsltCompileLocationPathPattern.
+	 */
+	element->priority = 0;
 	xsltCompileLocationPathPattern(ctxt);
 	if (ctxt->error) {
 	    xsltTransformError(NULL, style, node,
@@ -1775,42 +1807,36 @@ xsltCompilePattern(const xmlChar *pattern, xmlDocPtr doc,
 	/*
 	 * Set-up the priority
 	 */
-	if (((element->steps[0].op == XSLT_OP_ELEM) ||
-	     (element->steps[0].op == XSLT_OP_ATTR)) &&
-	    (element->steps[0].value != NULL) &&
-	    (element->steps[1].op == XSLT_OP_END)) {
-	    element->priority = 0;
-#if 0
-	} else if ((element->steps[0].op == XSLT_OP_ROOT) &&
-		   (element->steps[1].op == XSLT_OP_END)) {
-	    element->priority = 0;
-#endif
-	} else if ((element->steps[0].op == XSLT_OP_PI) &&
-		   (element->steps[0].value != NULL) &&
-		   (element->steps[1].op == XSLT_OP_END)) {
-	    element->priority = 0;
-	} else if ((element->steps[0].op == XSLT_OP_ATTR) &&
-		   (element->steps[0].value2 != NULL) &&
-		   (element->steps[1].op == XSLT_OP_END)) {
-	    element->priority = -0.25;
-	} else if ((element->steps[0].op == XSLT_OP_NS) &&
-		   (element->steps[0].value != NULL) &&
-		   (element->steps[1].op == XSLT_OP_END)) {
-	    element->priority = -0.25;
-	} else if ((element->steps[0].op == XSLT_OP_ATTR) &&
-		   (element->steps[0].value == NULL) &&
-		   (element->steps[0].value2 == NULL) &&
-		   (element->steps[1].op == XSLT_OP_END)) {
-	    element->priority = -0.5;
-	} else if (((element->steps[0].op == XSLT_OP_PI) ||
-		    (element->steps[0].op == XSLT_OP_TEXT) ||
-		    (element->steps[0].op == XSLT_OP_ALL) ||
-		    (element->steps[0].op == XSLT_OP_NODE) ||
-		    (element->steps[0].op == XSLT_OP_COMMENT)) &&
-		   (element->steps[1].op == XSLT_OP_END)) {
-	    element->priority = -0.5;
-	} else {
-	    element->priority = 0.5;
+	if (element->priority == 0) {	/* if not yet determined */
+	    if (((element->steps[0].op == XSLT_OP_ELEM) ||
+		 (element->steps[0].op == XSLT_OP_ATTR) ||
+		 (element->steps[0].op == XSLT_OP_PI)) &&
+		(element->steps[0].value != NULL) &&
+		(element->steps[1].op == XSLT_OP_END)) {
+		;	/* previously preset */
+	    } else if ((element->steps[0].op == XSLT_OP_ATTR) &&
+		       (element->steps[0].value2 != NULL) &&
+		       (element->steps[1].op == XSLT_OP_END)) {
+			element->priority = -0.25;
+	    } else if ((element->steps[0].op == XSLT_OP_NS) &&
+		       (element->steps[0].value != NULL) &&
+		       (element->steps[1].op == XSLT_OP_END)) {
+			element->priority = -0.25;
+	    } else if ((element->steps[0].op == XSLT_OP_ATTR) &&
+		       (element->steps[0].value == NULL) &&
+		       (element->steps[0].value2 == NULL) &&
+		       (element->steps[1].op == XSLT_OP_END)) {
+			element->priority = -0.5;
+	    } else if (((element->steps[0].op == XSLT_OP_PI) ||
+		       (element->steps[0].op == XSLT_OP_TEXT) ||
+		       (element->steps[0].op == XSLT_OP_ALL) ||
+		       (element->steps[0].op == XSLT_OP_NODE) ||
+		       (element->steps[0].op == XSLT_OP_COMMENT)) &&
+		       (element->steps[1].op == XSLT_OP_END)) {
+			element->priority = -0.5;
+	    } else {
+		element->priority = 0.5;
+	    }
 	}
 #ifdef WITH_XSLT_DEBUG_PATTERN
 	xsltGenericDebug(xsltGenericDebugContext,
